#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'environment_information/class/add.rb'
# =========================================================================== #
module EnvironmentInformation

class EnvironmentInformation # === EnvironmentInformation::EnvironmentInformation

  # ========================================================================= #
  # === add                                                         (add tag)
  #
  # This method can be used to add individual entries to the main array.
  #
  # The entries will have two variables:
  #
  #   1) the name of the component (normally the program)
  #   2) the version associated with that component
  #
  # If a Symbol is given then we may use this as a "pointer" towards
  # an array of registered programs.
  #
  # Note that (1) will be converted into a Symbol if it is a String.
  #
  # If the second entry (2) is nil, then the version will be inferred
  # at "runtime" within the method display(). Otherwise display()
  # will simply use the passed version (which should be a String in
  # that case, then).
  #
  # Before we can add an entry to the main dataset, we have to check
  # whether it is a registered entry altogether. If it is not
  # registered, the user has to be notified about this fact.
  # This notification will ONLY happen if the associated version is nil;
  # if a specific version is supplied then this method will assume
  # that the program should be displayed anyway.
  #
  # An Array can also be supplied to this method, if necessary.
  # ========================================================================= #
  def add(
      i, version_to_use = nil
    )
    if i.is_a?(String) and i.include?(',')
      i = i.split(',')
    elsif i.is_a?(String) and i.include?('-')
      i = i.dup if i.frozen?
      i.delete!('-')
    end
    case i
    # ======================================================================= #
    # === :everything
    #
    # This entry point is meant for simply adding everything.
    # ======================================================================= #
    when :everything
      i = return_every_registered_component
    # ======================================================================= #
    # === :linux_kernel
    # ======================================================================= #
    when :linux_kernel
      i = :linux
    # ======================================================================= #
    # === :RAM
    # ======================================================================= #
    when :RAM
      i = :ram # Just the "alias" to the real entry.
    # ======================================================================= #
    # === :all_xorg_components
    # ======================================================================= #
    when :all_xorg_components
      i = return_all_xorg_components
    # ======================================================================= #
    # === envi --science
    # ======================================================================= #
    when :science_cluster
      i = ARRAY_SCIENCE_CLUSTER
     # ======================================================================= #
     # === envi --lfs_core_programs
     # ======================================================================= #
     when :lfs,
          :lfs_core_programs
      i = ::EnvironmentInformation.lfs_core_programs?
    end
    if i.is_a? Array
      i.each {|entry, version_to_use| add(entry, version_to_use) }
    else
      unless @array_display_these_components.map {|entry| entry.first }.include? i 
        @array_display_these_components << [i, version_to_use]
        #@array_display_these_components.flatten!
      end
    end
  end; alias update_version_for         add # === add
       alias append                     add # === append
       alias append_this_to_main_string add # === append_this_to_main_string
       alias show_these_entries         add # === show_these_entries

end; end