#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === EnvironmentInformation::EnvironmentInformation
#
# The primary purpose of this class is to provide information about
# the computer environment ruby is running on. Obviously ruby must
# be available in order for this class (this program) to work.
#
# By default, class EnvironmentInformation will run in
# "commandline mode", that is, it will output information
# onto the console/terminal.
#
# The dataset is of course also available internally, though, so you
# can query this information and display it e. g. in a traditional
# GUI or WWW setting, like for a webpage.
#
# =========================================================================== #
# A complete usage example for the main class follows next:
#
#   require 'environment_information'
#
#   _ = EnvironmentInformation::EnvironmentInformation.new(:do_not_run_yet)
#   _.disable_colours
#   _.be_silent
#   _.run
#   _.string
#
# =========================================================================== #
# require 'environment_information/class/initialize.rb'
# =========================================================================== #
require 'environment_information/toplevel_methods/return_version_of_this_program.rb'
require 'environment_information/toplevel_methods/replay_from_the_stored_file.rb'
require 'environment_information/class/add.rb'
require 'environment_information/class/menu.rb'
require 'environment_information/class/misc.rb'
require 'environment_information/class/opn.rb'
require 'environment_information/class/register_sigint.rb'
require 'environment_information/class/reset.rb'
require 'environment_information/class/report.rb'
require 'environment_information/class/run.rb'

module EnvironmentInformation

class EnvironmentInformation # === EnvironmentInformation::EnvironmentInformation

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      commandline_arguments = nil,
      run_already           = true
    )
    register_sigint
    reset
    # ======================================================================= #
    # Next, we handle special run-mode variants.
    # ======================================================================= #
    case run_already
    when :do_show_everything
      do_show_everything
      run_already = true
    end
    # ======================================================================= #
    # And set @run_already, so that menu() can overrule it.
    # ======================================================================= #
    @run_already = run_already
    set_commandline_arguments(
      commandline_arguments
    )
    # ======================================================================= #
    # === Handle blocks
    #
    # Next blocks will be handled. This must come after the reset() method.
    # ======================================================================= #
    if block_given?
      yielded = yield
      case yielded # case tag
      # ===================================================================== #
      # === :do_not_use_rbt
      # ===================================================================== #
      when :do_not_use_rbt
        @may_we_try_to_use_rbt = false
      # ===================================================================== #
      # === :disable_colours
      # ===================================================================== #
      when :disable_colours
        disable_colours
      # ===================================================================== #
      # === :disable_colours_and_do_not_store_into_a_local_file
      # ===================================================================== #
      when :disable_colours_and_do_not_store_into_a_local_file
        disable_colours
        do_not_save_anything
      # ===================================================================== #
      # === :do_not_run_yet
      # ===================================================================== #
      when :do_not_run_yet
        @run_already = false
      else
        # =================================================================== #
        # Handle Hashes given in the block.
        # =================================================================== #
        if yielded.is_a? Hash
          # ================================================================= #
          # === :use_n_tokens
          # ================================================================= #
          if yielded.has_key? :use_n_tokens
            set_use_n_tokens(yielded.delete(:use_n_tokens))
          # ================================================================= #
          # === :n_tokens
          # ================================================================= #
          elsif yielded.has_key? :n_tokens
            set_use_n_tokens(yielded.delete(:n_tokens))
          end
          # ================================================================= #
          # === :use_colours
          # ================================================================= #
          if yielded.has_key? :use_colours
            set_use_colours(yielded.delete(:use_colours))
          end
          # ================================================================= #
          # === :be_silent
          # ================================================================= #
          if yielded.has_key? :be_silent
            set_be_silent(yielded.delete(:be_silent))
          end
          # ================================================================= #
          # === :show_ruby_stuff
          # ================================================================= #
          if yielded.has_key? :show_ruby_stuff
            if yielded.delete(:show_ruby_stuff) == false
              dont_show_ruby_stuff
            end
          end
        end
      end
    end
    # ======================================================================= #
    # Add the ruby components next.
    # ======================================================================= #
    consider_adding_the_ruby_components # ← Must come before menu().
    # ======================================================================= #
    # Invoke menu() to query for the commandline arguments.
    # ======================================================================= #
    menu
    run if @run_already
  end

  # ========================================================================= #
  # === EnvironmentInformation::EnvironmentInformation[]
  # ========================================================================= #
  def self.[](i = '', &block)
    new(i, &block)
  end

end

# =========================================================================== #
# === EnvironmentInformation.run_the_main_class
# =========================================================================== #
def self.run_the_main_class(i = ARGV, &block)
  ::EnvironmentInformation::EnvironmentInformation.new(i, &block)
end; self.instance_eval { alias result run_the_main_class } # === EnvironmentInformation.result
     self.instance_eval { alias show   run_the_main_class } # === EnvironmentInformation.show
     self.instance_eval { alias new    run_the_main_class } # === EnvironmentInformation.new
     self.instance_eval { alias []     run_the_main_class } # === EnvironmentInformation[]

# =========================================================================== #
# === EnvironmentInformation.environment_information?
#
# This method will return a very long string containing all
# information about the locally installed/available programs.
# =========================================================================== #
def self.environment_information?
  ::EnvironmentInformation::EnvironmentInformation.new(:full_be_silent).stringified
end

end

if __FILE__ == $PROGRAM_NAME
  EnvironmentInformation::EnvironmentInformation.new(ARGV)
end # class