#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'environment_information/class/menu.rb'
# =========================================================================== #
require 'environment_information/class/colours.rb'
require 'environment_information/class/help.rb'
require 'environment_information/class/html.rb'
require 'environment_information/class/ruby.rb'

module EnvironmentInformation

class EnvironmentInformation # === EnvironmentInformation::EnvironmentInformation

  # ========================================================================= #
  # === menu                                                       (menu tag)
  #
  # This method constitutes the "menu" interface for class
  # EnvironmentInformation. It usually deals with the
  # commandline-given arguments stored in ARGV.
  # ========================================================================= #
  def menu(
      i = @commandline_arguments
    )
    if i.is_a? Array
      i.each {|entry| menu(entry) }
    else
      # ===================================================================== #
      # Handle the given commandline arguments via this big menu.
      # ===================================================================== #
      case i # case tag
      # ===================================================================== #
      # === envi --really-all
      #
      # This entry point is for really showing everything.
      # ===================================================================== #
      when /^-?-?REALLY(_|-)?ALL$/i,
           /^-?-?compare(_|-)?program(_|-)?version$/i,
           /^-?-?compare(_|-)?to(_|-)?program(_|-)?versions$/i,
           /^-?-?do(_|-)?compare(_|-)?program(_|-)?version$/i,
           /^-?-?really(_|-)?everything$/i
        do_show_everything
        do_compare_the_program_version
      # ===================================================================== #
      # === envi --no-save
      #
      # Skip saving into a local file through this entry point.
      #
      # Invocation example:
      #
      #   envi --lfs --no-save
      #
      # ===================================================================== #
      when /^-?-?no(_|-)?save$/i
        do_not_save_anything
      # ===================================================================== #
      # === envi --be_silent_no_colours
      #
      # Not sure why the following entry point exists, but I will retain it
      # for the time being.
      # ===================================================================== #
      when /^-?-?be(_|-)?silent(_|-)?no(_|-)?colours$/
        disable_colours
        do_not_display_the_result
        do_show_everything
      # ===================================================================== #
      # === envi --show-components?
      # ===================================================================== #
      when /^-?-?show(-|_)?components\??$/i
        e "#{true_rev}The following components will be shown on a default run:"
        e
        show_these_components?.each {|this_component|
          if this_component.is_a? Array
            this_component = this_component.first
          end
          e "  - #{steelblue(this_component)}#{true_rev}"
        }
        e
        exit
      # ===================================================================== #
      # This entry point is mostly for internal use, e. g. to pass
      # in Symbols to menu().
      # ===================================================================== #
      when :be_silent_no_colours_everything,
           :be_silent_and_no_colours,
           :be_silent_no_colours,
           :full_be_silent
        disable_colours
        do_show_everything
        do_not_display_the_result
      # ===================================================================== #
      # === envi --be_silent
      # ===================================================================== #
      when /^-?-?be(_|-)?silent$/i # === envi --be-silent
        do_not_display_the_result
        do_show_everything
      # ===================================================================== #
      # === envi --work-on-programs-directory-only
      # ===================================================================== #
      when /^-?-?work(-|_)?on(-|_)?programs(-|_)?directory(-|_)?only$/i,
           /^-?-?work(-|_)?on(-|_)?the(-|_)?programs(-|_)?directory(-|_)?only$/i
        work_on_the_programs_directory_only
      # ===================================================================== #
      # === envi --do-save
      # ===================================================================== #
      when /^save$/i,
           /^-?-?do(_|-)?save$/i
        @store_the_results_into_local_files = true
      # ===================================================================== #
      # === :do_not_run_yet
      #
      # Invocation example:
      #
      #   envi --do-not-run-yet
      #
      # ===================================================================== #
      when /^-?-?-do(-|_)?not(-|_)?run(-|_)?yet$/i, # === envi --do-not-run-yet
           :do_not_run_yet,
           :dont_run_yet,
           :default # This is the default variant.
        @run_already = false
      # ===================================================================== #
      # === envi --asciitable
      # ===================================================================== #
      when /^-?-?asciitable$/i,
           'table','1'
        use_ascii_table if ASCIITABLE_IS_AVAILABLE
      # ===================================================================== #
      # === envi --read-from-this-file=/Depot/j/display_these_programs.md
      # === envi --file=/Depot/j/display_these_programs.md
      # === envi --input-from=/Depot/j/display_these_programs.md
      # ===================================================================== #
      when /^-?-?read(-|_)?from(-|_)?this(-|_)?file=(.+)$/i, # <- And this has $4
           /^-?-?file=(.+)$/i, # <- This has only $1
           /^-?-?input(-|_)?from=(.+)$/i # <- This has $2
        _ = $1.to_s.dup
        _ = $2.to_s.dup if $2
        _ = $4.to_s.dup if $4
        clear_old_dataset
        set_use_this_as_main_input(
          load_dataset_from_this_file(_)
        )
      # ===================================================================== #
      # === envi --no-yaml-file
      # ===================================================================== #
      when /^-?-?no(-|_)?yaml(-|_)?file$/i,
           /^-?-?no(-|_)?local(-|_)?file$/i
        @store_the_results_into_local_files = false
      # ===================================================================== #
      # === envi noruby
      # ===================================================================== #
      when /^-?-?noruby$/i,
           'show_no_ruby',
           'classic',
           'naked',
           '3',
           'minimal'
        show_no_ruby
      # ===================================================================== #
      # === envi --show-all
      # ===================================================================== #
      when /^-?-?show(-|_)?all$/i,
           /^-?-?all$/i,
           /^-?-?full$/i,
           /^-?-?everything$/i,    # === envi --everything
           /^-?-?information$/i,
           'f',
           '4'
        do_show_everything # <- Just to be sure.
      # ===================================================================== #
      # === envi --short
      # ===================================================================== #
      when /^-?-?short$/i,
           /^-?-?display(-|_)?short(-|_)?format$/i,
           /^-?-?do(-|_)?display(-|_)?in(-|_)?a(-|_)?short(-|_)?format$/i
        do_display_in_a_short_format
      # ===================================================================== #
      # === envi --xorg-components
      #
      # This entry point is for when the user only wants to show the
      # xorg-components on the local computer system.
      # ===================================================================== #
      when /^-?-?xorg(_|-)?components$/i,
           /^-?-?xorg$/i,
           /^-?-?show(_|-)?xorg(_|-)?components$/i
        do_show_only_the_xorg_components
      # ===================================================================== #
      # === :be_silent
      # ===================================================================== #
      when /^-?-?be(_|-)?silent$/i,
           :be_silent # We will be silent, but we will still show everything.
        do_not_display_the_result
        do_show_everything
      # ===================================================================== #
      # === envi --compare_programs
      #
      # This entry point allows us to compare the program versions with
      # the one registered in RBT.
      # ===================================================================== #
      when *ARRAY_COMPARE_PROGRAM_VERSIONS
        do_compare_the_program_version
      # ===================================================================== #
      # === envi --additional=php
      #
      # This entry point allows us to show additional programs.
      #
      # Another usage example:
      #
      #   envi --additional=php,python,perl
      #
      # ===================================================================== #
      when /^-?-?additional=(.+)$/i,
           /^-?-?add=(.+)$/i
        _ = $1.to_s.dup
        add(_)
      # ===================================================================== #
      # === envi --os?
      # ===================================================================== #
      when /^-?-?OS\??$/i,
           /^-?-?show(-|_)?only(-|_)?the(-|_)?operating(-|_)?system$/i
        do_show_only_the_operating_system
        show_no_ruby
      # ===================================================================== #
      # === envi --sort-alphabetically
      #
      # This entry point can also be combined, such as in:
      #
      #   envi --show-everything --sort-alphabetically
      #
      # ===================================================================== #
      when /^-?-?sort(-|_)?alphabetically$/i,
           /^-?-?sort$/i,
           /^-?-?alphabetically$/i,
           /^-?-?alphabet$/i
        enable_sort_alphabetically
      # ===================================================================== #
      # === :do_show_everything
      #
      # An alternative way to make use of this entry point is via:
      #
      #   envi --do-show-everything
      #   envi --show-everything
      #
      # ===================================================================== #
      when :do_show_everything,
           :show_really_everything,
           /^-?-?do(_|-)?show(_|-)?everything$/i,
           /^-?-?show(-|_)?everything$/i,
           /^-?-?show(-|_)?full(-|_)?information$/i,
           /^-?-?everything$/i,
           /^-?-?rall$/i,
           'ALL'
        do_show_everything
        @run_already = true
      # ===================================================================== #
      # === envi --registered-components?
      # ===================================================================== #
      when /^-?-?registered(_|-)?components\??$/i,
           /^-?-?available\??$/i
        do_not_run_already
        show_the_registered_components
        @do_exit_the_program = true
      # ===================================================================== #
      # === envi --use-these-programs=bash,binutils,bison,yacc,bzip2,coreutils,diff,find,gawk,gcc,grep,gzip,linux,make,m4,patch,perl,python,sed,tar,makeinfo,xz
      # === envi --use-these-programs=:lfs
      #
      # This entry point can be used to display a certain subset of
      # programs, at the user's discretion.
      # ===================================================================== #
      when /^-?-?use(-|_)?these(-|_)?programs=(.+)$/i # === $3
        # =================================================================== #
        # First clear (aka reset) the main dataset.
        # =================================================================== #
        clear_old_dataset
        match = $3.to_s.dup
        add(match)
      # ===================================================================== #
      # === envi --clear
      #
      # This entry point will simply clear the old dataset.
      #
      # This can be used to only display a few components, such as:
      #
      #   envi --clear --python --ruby --perl
      #
      # ===================================================================== #
      when /^-?-?clear$/i,
           /^-?-?clear(_|-)?old(_|-)?dataset$/i
        clear_old_dataset
      # ===================================================================== #
      # === envi --sinatra
      # ===================================================================== #
      when /^-?-?sinatra$/i,
           /^-?-?www$/i
        start_the_sinatra_interface
      # ===================================================================== #
      # === envi --generate-html-file
      # ===================================================================== #
      when /^-?-?generate(-|_)?html(-|_)?file$/i,
           /^-?-?html$/i # envi --HTML
        @generate_a_html_file = true
        set_runmode_html # Set it here again, just in case.
        do_show_everything
      # ===================================================================== #
      # === envi --n_entries
      # ===================================================================== #
      when /^-?-?n(_|-)?entries$/i
        show_n_registered_entries # === envi --n_entries
        @do_exit_the_program = true
      # ===================================================================== #
      # === envi --open
      # ===================================================================== #
      when /^-?-?open$/i,
           /^-?-?edit$/i
        open_this_file_in_editor
      # ===================================================================== #
      # === envi --lfs
      # ===================================================================== #
      when /^-?-?lfs$/i,
           /^-?-?lfs(_|-)?core(_|-)?programs$/i
        clear_old_dataset
        add(:lfs_core_programs)
      # ===================================================================== #
      # === envi openssl
      # ===================================================================== #
      when /^-?-?openssl$/,
           'ssl'
        add(:openssl)
      # ===================================================================== #
      # === envi --help
      # ===================================================================== #
      when /help/i
        @show_help = true
      # ===================================================================== #
      # === envi --show-remote-url
      # ===================================================================== #
      when /^-?-?show(_|-)?remote(_|-)?url$/i
        do_report_the_remote_urls
      # ===================================================================== #
      # === envi --science
      # ===================================================================== #
      when /^-?-?science$/
        clear_old_dataset
        add(:science_cluster)
      # ===================================================================== #
      # === envi --nocolours
      #
      # This entry point can be used to disable usage of colours.
      # ===================================================================== #
      when /^-?-?nocolours/,'2',
           /^-?-?no(-|_)?colou?rs$/i,
           /^-?-?nocol$/i,
           /^-?-?disable(-|_)?colou?rs$/i, # === envi --really-all --disable-colours
           :disable_colours
        disable_colours
      # ===================================================================== #
      # === envi --use-colours?
      # ===================================================================== #
      when /^-?-?use(-|_)?colou?rs\??$/i
        e 'Will colours be used? '+
          ::EnvironmentInformation.use_colours?.to_s
        do_not_run
        @do_exit_the_program = true
      # ===================================================================== #
      # === envi --version
      # ===================================================================== #
      when *ARRAY_VERSION
        report_version
      # ===================================================================== #
      # === envi --gui
      # ===================================================================== #
      when /^-?-?gui$/i,
           /^-?-?gtk$/i,
           /^-?-?start(-|_)?gtk$/i
        start_gtk_component
      # ===================================================================== #
      # === envi --help
      # ===================================================================== #
      when /^-?-?help$/i
        show_help
      else # else tag
        # =================================================================== #
        # It is better to report that the given commandline-input as
        # not found, so this will be reported. Before we can do so, we
        # will first check whether the user wanted to pass some
        # existing/registered component nonetheless.
        #
        # This clause can be checked via something like this:
        #
        #   envi --bash
        #
        # =================================================================== #
        if is_a_registered_component?(i.delete('-'))
          add(
            i.delete('-')
          )
        else
          e 'Unhandled command input (in file environment_information/class/menu.rb): '
          e
          e "  #{sfancy(i)}"
          e
        end
      end
    end
  end

end; end