#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'environment_information/constants/array_tracked_components.rb'
# EnvironmentInformation.is_this_program_included?
# =========================================================================== #
require 'yaml'
require 'environment_information/constants/file_constants.rb'

module EnvironmentInformation

  # ========================================================================= #
  # === ARRAY_LFS_CORE_PROGRAMS
  #
  # All the LFS core programs are registered here in this Array.
  # ========================================================================= #
  ARRAY_LFS_CORE_PROGRAMS =
    YAML.load_file(FILE_ARRAY_LFS_CORE_PROGRAMS)

  # ========================================================================= #
  # === ARRAY_XORG_COMPONENTS                                      (xorg tag)
  #
  # All xorg-components will be collected here. Sort alphabetically, please,
  # and keep the entries downcased.
  #
  # xf86dga is an xorg-component, but since as of October 2018 it has
  # been removed and will no longer be checked - it seems to have been
  # abandoned, more or less.
  # ========================================================================= #
  ARRAY_XORG_COMPONENTS = YAML.load_file(FILE_ARRAY_TRACKED_XORG_COMPONENTS)

  # ========================================================================= #
  # === EnvironmentInformation::ARRAY_TRACKED_PROGRAMS
  #
  # This Array keeps track of which programs are tracked by the
  # EnvironmentInformation project. That listing will include ALL
  # registered entries, with a corresponding entry under the
  # directory individual_components/.
  #
  # All entries of this Array should be downcased.
  #
  # Keep this Array alphabetically sorted, please, even though this is
  # not an absolute necessity; it just simplifies adding and removing
  # entries in the long run.
  #
  # Since as of February 2020 this is maintained in a yaml file.
  # ========================================================================= #
  if File.exist? FILE_ARRAY_TRACKED_PROGRAMS
    ARRAY_TRACKED_PROGRAMS = YAML.load_file(FILE_ARRAY_TRACKED_PROGRAMS)
  else
    puts 'No file could be found at '+FILE_ARRAY_TRACKED_PROGRAMS+'.'
    ARRAY_TRACKED_PROGRAMS = [] # Default in this case.
  end

  # ========================================================================= #
  # === ARRAY_DEFAULT_PROGRAMS_ON_LINUX
  #
  # This Array keeps track of the default programs on a linux computer.
  # ========================================================================= #
  ARRAY_DEFAULT_PROGRAMS_ON_LINUX =
    YAML.load_file(FILE_ARRAY_DEFAULT_PROGRAMS_ON_LINUX)

  # ========================================================================= #
  # === ARRAY_TRACKED_NON_PROGRAMS
  #
  # The entry rubygems_installation_directory depends on the executable
  # called "gem", which is part of ruby.
  #
  # This Array has also been called "registered extra functionality" in
  # the past. The entries here are special in one way or another.
  # Usually these entries are NOT standalone-programs, which is
  # why they are tracked separately.
  # ========================================================================= #
  ARRAY_TRACKED_NON_PROGRAMS =
    YAML.load_file(FILE_ARRAY_TRACKED_NON_PROGRAMS)

  # ========================================================================= #
  # === ARRAY_SCIENCE_CLUSTER                                   (science tag)
  #
  # Science-specific applications are gathered in this array.
  # ========================================================================= #
  ARRAY_SCIENCE_CLUSTER = 
    YAML.load_file(FILE_ARRAY_SCIENCE_CLUSTER)

  # ========================================================================= #
  # === EnvironmentInformation.science_cluster?
  # ========================================================================= #
  def self.science_cluster?
    ARRAY_SCIENCE_CLUSTER
  end

  # ========================================================================= #
  # === ARRAY_LFS_TEMPORARY_PROGRAMS
  #
  # All programs that are temporary, e. g. for enabling the bootstrapping
  # of the compiler, are stored in the following Array.
  # ========================================================================= #
  ARRAY_LFS_TEMPORARY_PROGRAMS = %i(
    binutils
    gcc
    linux_API_headers
    glibc
    libstdc++
    binutils
    tcl
    expect
    dejagnu
    m4
    ncurse
    bash
    bison
    bzip2
    coreutils
    diffutils
    file
    findutils
    gawk
    gettext
    grep
    gzip
    make
    patch
    perl
    python
    sed
    tar
    texinfo
    xz
  )

  # ========================================================================= #
  # === EnvironmentInformation.tracked_programs?
  #
  # Simpler toplevel-method to find out which programs are registered
  # within the EnvironmentInformation project.
  # ========================================================================= #
  def self.tracked_programs?
    ARRAY_TRACKED_PROGRAMS
  end

  # ========================================================================= #
  # === EnvironmentInformation.xorg_components?
  # ========================================================================= #
  def self.xorg_components?
    ARRAY_XORG_COMPONENTS
  end; self.instance_eval { alias return_all_xorg_components_as_array xorg_components? } # === EnvironmentInformation.return_all_xorg_components_as_array
       self.instance_eval { alias return_all_xorg_components_as_array xorg_components? } # === EnvironmentInformation.return_all_xorg_components_as_array

  # ========================================================================= #
  # === EnvironmentInformation.tracked_non_programs?
  #
  # Simpler toplevel-method to find out which non-programs are registered
  # within the EnvironmentInformation project.
  # ========================================================================= #
  def self.tracked_non_programs?
    ARRAY_TRACKED_NON_PROGRAMS
  end; self.instance_eval { alias registered_extra_functionality tracked_non_programs? } # === EnvironmentInformation.registered_extra_functionality

  # ========================================================================= #
  # === EnvironmentInformation.n_subcommands?
  #
  # Return how many subcommands are available/registered. These are
  # the tracked programs; and the xorg-components.
  #
  # The non-programs will not be gathered by this method, on purpose; they
  # are not standalone programs after all.
  # ========================================================================= #
  def self.n_subcommands?
    ARRAY_TRACKED_PROGRAMS.size+
    ARRAY_XORG_COMPONENTS.size
  end

  # ========================================================================= #
  # === EnvironmentInformation.lfs_core_programs?
  # ========================================================================= #
  def self.lfs_core_programs?
    ARRAY_LFS_CORE_PROGRAMS
  end

  # ========================================================================= #
  # === EnvironmentInformation.is_this_program_included?
  #
  # This will include the two Arrays ARRAY_TRACKED_PROGRAMS and
  # ARRAY_XORG_COMPONENTS, but it will NOT include ARRAY_TRACKED_NON_PROGRAMS.
  # ========================================================================= #
  def self.is_this_program_included?(i)
    (
      ARRAY_TRACKED_PROGRAMS +
      ARRAY_XORG_COMPONENTS
    ).flatten.include? i.to_sym # Need a symbol.
  end; self.instance_eval { alias is_this_a_registered_program? is_this_program_included? } # === ::EnvironmentInformation.is_this_a_registered_program?

  # ========================================================================= #
  # === EnvironmentInformation.is_this_component_included?
  #
  # This is a more extensive check than .is_this_program_included?()
  # ========================================================================= #
  def self.is_this_component_included?(i)
    array = (
              ARRAY_TRACKED_PROGRAMS +
              ARRAY_XORG_COMPONENTS  +
              ARRAY_TRACKED_NON_PROGRAMS
            )
    array.flatten.include? i.to_sym # Need a symbol.
  end

end

if __FILE__ == $PROGRAM_NAME
  pp EnvironmentInformation.lfs_core_programs?
end