#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'environment_information/toplevel_methods/return_version_of_this_program.rb'
# EnvironmentInformation.return_version_of_this_program
# =========================================================================== #
require 'yaml'
require 'environment_information/colours/colours.rb'
require 'environment_information/constants/array_tracked_components.rb'
require 'environment_information/constants/error_line.rb'
require 'environment_information/constants/misc.rb'
require 'environment_information/constants/newline.rb'
require 'environment_information/toplevel_methods/prefix_to_use.rb'
require 'environment_information/requires/require_the_individual_components.rb'

module EnvironmentInformation

  # ========================================================================= #
  # === @debug
  #
  # This variable can be used to control whether we debug the project
  # or not. It should be false by default (and upon re-destribution of
  # the project to other users).
  # ========================================================================= #
  @debug = false

  # ========================================================================= #
  # === @query_to_use_for_the_individual_components
  # ========================================================================= #
  @query_to_use_for_the_individual_components = YAML.load_file(
    FILE_QUERY_TO_USE_FOR_THE_INDIVIDUAL_COMPONENTS
  )

  # ========================================================================= #
  # === EnvironmentInformation.return_version_of_this_program
  #
  # This is the general method that will return the version of a particular
  # program at hand.
  #
  # The method must be able to deal with using pkg-config, but also
  # querying some program's version via --version, via the commandline,
  # Furthermore, some programs may require an ad-hoc fix.
  # ========================================================================= #
  def self.return_version_of_this_program(
      this_program,
      prefix_to_use = @prefix_to_use
    )
    prefix_to_use = prefix_to_use.dup if prefix_to_use
    version = nil # ← This is the default.
    if this_program.is_a? Array
      this_program = this_program.flatten.first
    end
    # Next define a few aliases.
    case this_program.to_s
    when 'diff'
      this_program = 'diffutils'
    when 'yacc'
      this_program = 'bison'
    end
    if ARRAY_TRACKED_NON_PROGRAMS.include? this_program
      return ::EnvironmentInformation.send(this_program)
    elsif @query_to_use_for_the_individual_components.has_key? this_program.to_sym
      use_this_command = @query_to_use_for_the_individual_components[this_program.to_sym]
      case use_this_command
      # ======================================================================= #
      # === :mate_desktop
      #
      # This entry is special.
      # ======================================================================= #
      when :mate_desktop
        return ::RBT.return_mate_desktop_version_array
      # ======================================================================= #
      # === :pkgconfig
      # ======================================================================= #
      when :pkgconfig
        # =================================================================== #
        # In this case it must be generic.
        # =================================================================== #
        cmd = "pkg-config --modversion #{this_program} #{ERROR_LINE}"
        version = `#{cmd}`
        version = COMMAND_NOT_FOUND if version.include? NOT_FOUND_IN_PKG_CONFIG_SEARCH_PATH
      # ======================================================================= #
      # === :custom_gtk2
      # ======================================================================= #
      when :custom_gtk2
        cmd = "pkg-config --modversion gtk+-2.0 #{ERROR_LINE}"
        version = `#{cmd}`
        version = COMMAND_NOT_FOUND if version.include? NOT_FOUND_IN_PKG_CONFIG_SEARCH_PATH
      # ======================================================================= #
      # === :custom_mpc
      #
      # We rely on the header called mpc.h.
      # ======================================================================= #
      when :custom_mpc
        target_file = '/usr/include/mpc.h'
        if File.exist? target_file
          dataset = File.read(target_file)
          use_this_regex = /MPC_VERSION_STRING "([0-9\.]+)"$/
          dataset = dataset.scan(use_this_regex).flatten
          version = dataset.first 
        end
      # ======================================================================= #
      # === :custom_boost
      # ======================================================================= #
      when :custom_boost
        target_file = '/usr/include/boost/version.hpp'

        if File.exist?('/System/Index/include/boost/version.hpp') and
          !File.exist?(target_file) # Custom fix.
          target_file = '/System/Index/include/boost/version.hpp'
        end
        if File.exist? target_file
          dataset = File.readlines(target_file)
          version = dataset.select {|line|
            line.include? '#define BOOST_LIB_VERSION'
          }.first.sub(/#define BOOST_LIB_VERSION/,'').strip.delete('"').tr('_','.')
          if version.count('.') < 2
            version << '.0'
          end
        else
          register_this_component_is_missing(:boost)
        end
      # ======================================================================= #
      # === :version
      #
      # This entry point is typically for "program_name --version"
      # entries. Some of them require custom modifications.
      # ======================================================================= #
      when :version
        # =================================================================== #
        # Next enable support for AppDir layout, where programs will
        # ultimately reside in the same directory.
        # =================================================================== #
        unless prefix_to_use.empty?
          prefix_to_use << "#{this_program.to_s.capitalize}/Current/bin/"
        end
        cmd = "#{prefix_to_use}#{this_program} --version #{ERROR_LINE}"
        if @debug
          e Colours.crimson('DEBUG: ')+
            Colours.steelblue(cmd)
        end
        # =================================================================== #
        # === Run the system command
        #
        # Next run the system command:
        # =================================================================== #
        _ = `#{cmd}`.strip
        case this_program.to_sym
        when :bison,
             :libtool,
             :gcc,
             :gettext,
             :m4,
             :grep
          _ = _.split(N).first.split(' ').last
        when :screen
          # ================================================================= #
          # Assume a String like this: "Screen version 4.08.00 (GNU) 05-Feb-20"
          # ================================================================= #
          version = _.scan(/version (\d{1}\.?\d{1,2}\.?\d{1})/).flatten.first.to_s.strip
          # ================================================================= #
          # Must still correct the middle part.
          # ================================================================= #
          splitted = version.split('.')
          splitted[1][0,1] = '' # Kill the leading '0'.
          version = splitted.join('.')
          _ = nil
        when :php
          # ================================================================= #
          # This check has to be a bit more sophisticated, so we
          # use scan.
          #
          # Examples:
          #
          #   PHP 5.6.23 (cli) (built: Jun 24 2016 16:19:35)
          #   PHP 7.4.2 (cli) (built: Jan 21 2020 16:57:46) ( NTS )
          #
          # ================================================================= #
          result = _.scan(
            /PHP (\d{1,2}\.?\d{1,2}\.?\d{1,2}) \(cli\)/
          )
          _ = result.flatten.first.strip
        when :ruby
          _ = _.split(' ')[1]
          if _.include? 'p'
            _ = _[0 .. (_.index('p') - 1)]
          end
        end
        # =================================================================== #
        # The program could not be found.
        # =================================================================== #
        if _.include? NO_SUCH_FILE_OR_DIRECTORY
          _ = nil
          version = nil
        # =================================================================== #
        # Assume output like this:
        #
        #   This is perl 5, version 30, subversion 0 (v5.30.0) built for x86_64-linux-thread-multi
        #
        # =================================================================== #
        elsif _.include?('This is perl ')
          use_this_regex = /\(v(\d{1,2}.\d{1,2}.\d{1,2})\)/ # See: https://rubular.com/r/I1z0M5T9oNaVm4
          _ =~ use_this_regex
          _ = $1.to_s.dup
        # =================================================================== #
        # === Handle cmake-related errors
        # =================================================================== #
        elsif _.include?('Could not find CMAKE_ROOT')
          _ = nil
          version = nil
        # =================================================================== #
        # === Handle VIM next
        # =================================================================== #
        elsif _.include?('VIM - Vi IMproved ')
          _ = _.scan(
            /VIM - Vi IMproved (\d{1,2}\.?\d{1,2}) /
          ).flatten.first.to_s.dup
        # =================================================================== #
        # === Handle XZ utils next
        # =================================================================== #
        elsif _.include?('xz (XZ Utils) ')
          _ = _.scan(
            /xz \(XZ Utils\) (\d{1,2}\.?\d{1,2}\.?\d{1,2})/
          ).flatten.first.to_s
        elsif _.include? 'GNU tar'
          use_this_regex = /tar \(GNU tar\) (\d+\.?\d{2})/
          _ =~ use_this_regex
          _ = $1.to_s.dup
        elsif _.include? 'sed (GNU sed) '
          _ = _.scan(/sed \(GNU sed\) (\d+.\d+)/).flatten.first.to_s
        elsif _.include? N
          _ = _.split(N).first
        end if _
        if _
          _.sub!(/^GNU /,'') if _.start_with? 'GNU '
          _.sub!(/ built on linux-gnu./,'') if _.include? ' built on linux-gnu.'
          if _.include? ', version '
            # =============================================================== #
            # This is specifically for bash.
            #
            # Example:
            #   GNU bash, version 5.0.0(1)-release (x86_64-pc-linux-gnu)
            # =============================================================== #
            version = _.scan(
              /, version (\d{1,2}\.?\d{1,2}\.?\d{1,2})/
            ).flatten.first.strip
            if this_program == :bash
              version.chop!; version.chop! # Ad-hoc workaround.
            end
          elsif _.include? ' Version '
            version = _.scan(/\d{1,2}\.?\d{1,2}\.?\d{1,2}/).flatten.first.to_s
          elsif _.include? ' version '
            version = _.scan(/ version (\d{1,2}\.?\d{1,2}\.?\d{1,2})/).flatten.first.to_s
          else
            if _.include? ', '
              _ = _.split(',').first # Fix entries like: "Awk 5.0.1, API: 2.0"
            end
            # =============================================================== #
            # Assume a String like this, which has to be modified a bit:
            #
            #   "gnuplot 5.2 patchlevel 8"
            # 
            # =============================================================== #
            _.sub!(/ patchlevel /,'.') if _.include? ' patchlevel '
            # =============================================================== #
            # Else assume a result such as this:
            #
            #   bison (GNU Bison) 3.5.1
            #
            # =============================================================== #
            if _.include?(' ') and !_.include?(' (') # <- The second case will be for vim.
              version = _.split(' ').last.strip
            # =============================================================== #
            # The next clause is specifically for this:
            #
            #   file-5.38
            #
            # =============================================================== #
            elsif _.include?('-') and !_.include?(' (') 
              version = _.split('-').last.strip
            elsif _.include? ' ('
              # =============================================================== #
              # This must also handle cases such as: "xz (XZ Utils) 5.2.4"
              # =============================================================== #
              version = _.split(' (').last.to_s.strip
            else
              version = _
            end
            # =============================================================== #
            # Next specifically remove 'v' because "node --version" may
            # include that.
            # =============================================================== #
            version.delete!('v') if version.include? 'v'
          end
        end
      else # else tag
        cmd = "#{@prefix_to_use}#{use_this_command} #{ERROR_LINE}"
        first_line = nil
        # =================================================================== #
        # Next, run that system-command:
        # =================================================================== #
        result_of_system_cmd = `#{cmd}`
        if result_of_system_cmd.encoding.to_s == 'US-ASCII'
          result_of_system_cmd = result_of_system_cmd.force_encoding(UTF_ENCODING)
        end
        newline_splitted = result_of_system_cmd.split(N)
        if newline_splitted.first.include? 'hmmpgmd :: search a query against a database'
          newline_splitted.shift
        end
        # =================================================================== #
        # Handle the case where the .pc file could not be found.
        # =================================================================== #
        if result_of_system_cmd.include? NOT_FOUND_IN_PKG_CONFIG_SEARCH_PATH
          first_line = COMMAND_NOT_FOUND
          version = nil
        elsif result_of_system_cmd.include? COMMAND_NOT_FOUND
          first_line = COMMAND_NOT_FOUND
        elsif result_of_system_cmd.include? 'java version "'
          first_line = result_of_system_cmd.scan(
            /java version "(\d{1,2}\.?\d{1,2}\.?\d{1,2})/
          ).flatten.first.to_s.strip
        elsif result_of_system_cmd.include? 'KDE Frameworks: '
          first_line = result_of_system_cmd.scan(
            /KDE Frameworks: (\d{1,2}\.?\d{1,2}\.?\d{1,2})/
          ).flatten.first
        end
        if this_program == :ncurses
          first_line = result_of_system_cmd.split(' ').last.strip
          first_line = first_line.split('.')[0..-2].join('.')
        elsif this_program == :ffmpeg
          first_line = result_of_system_cmd.split(N).first.split(' ')[2]
        elsif result_of_system_cmd.include? 'LLVM version '
          first_line = result_of_system_cmd.scan(
            /LLVM version (\d{1,2}\.?\d{1,2}\.?\d{1,2})/
          ).flatten.first
        else
          first_line = newline_splitted.first if first_line.nil?
        end
        if first_line
          if first_line.include? '# '
            if first_line.start_with? '# '
              first_line.sub!(/# /,'')
            end
          elsif first_line.include? ' #'
            first_line = first_line[0 .. first_line.index(' #')].strip
          end
        end
        if first_line.include? COMMAND_NOT_FOUND
          version = nil
        elsif first_line.include? 'Version: ImageMagick '
          version = first_line.scan(
            /Version: ImageMagick (\d{1,2}\.?\d{1,2}\.?\d{1,2}-\d{1,2})/
          ).flatten.first.to_s.strip.tr('-','.')
        elsif first_line.include? 'HMMER'
          version = first_line.scan(
            /HMMER (\d{1,2}\.?\d{1,2}\.?\d{1,2})/
          ).flatten.first.to_s.strip
        # === "XTerm(353)"
        elsif first_line.include?('(') and first_line.include?(')')
          version = first_line.split('(').last.delete(')')
        elsif first_line.include? ' Version '
          version = first_line.scan(
            / Version (\d{1,2}\.?\d{1,2}\.?\d{1,2})/
          ).flatten.first.to_s.strip
        elsif first_line.include? ' version '
          version = first_line.scan(
            / version (\d{1,2}\.?\d{1,2}\.?\d{1,2})/
          ).flatten.first.to_s.strip
        else
          version = first_line
          if version.include? ' '
            version = version.split(' ').last
          end
          version.strip!
        end
        if version
          if version.include?('/')
            # =============================================================== #
            # Assume input such as:
            #
            #   nginx version: nginx/1.16.1
            #
            # =============================================================== #
            version = version.split('/').last.strip
          end
          if version.include? ' '
            version = version.split(' ').last # Always assume the last part in this case.
          end
        end
      end
    else
      e 'Not registered any key for the program: '+this_program.to_s
      e 'This is currently not allowed - please add this missing information.'
      exit
    end
    if version and version.include? 'n'
      version = version.dup if version.frozen?
      version.delete!('n') # Ad-hoc fix for "node --version".
    end
    result = version
    if result
      unless result.include? COMMAND_NOT_FOUND
        result = result.dup if result.frozen?
        result.strip!
      end
    end
    if result and (result == 'found' or result == 'foud')
      register_this_component_is_missing(this_program)
    elsif result and result.include?(COMMAND_NOT_FOUND)
      register_this_component_is_missing(this_program)
    # ======================================================================= #
    # Check whether pkg-config is available or not.
    # ======================================================================= #
    elsif result and (
            result.include?(PKGCONFIG_COMMAND_NOT_FOUND) or
            result.include?(NO_SUCH_FILE_OR_DIRECTORY)
          )
      register_this_component_is_missing(this_program)
    elsif result.nil?
      register_this_component_is_missing(this_program)
    end
    return result
  end; self.instance_eval { alias version_for? return_version_of_this_program } # === EnvironmentInformation,version_for?

end

if __FILE__ == $PROGRAM_NAME
  puts EnvironmentInformation.return_version_of_this_program(ARGV)
end # rb return_version_of_this_program.rb ruby