#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'environment_information/toplevel_methods/show_all_available_components.rb'
# EnvironmentInformation.show_all_available_components
# =========================================================================== #
require 'environment_information/constants/array_tracked_components.rb'
require 'environment_information/constants/regex.rb'
require 'environment_information/requires/require_the_individual_components.rb'
require 'environment_information/toplevel_methods/autogenerate_all_relevant_methods.rb'
require 'environment_information/toplevel_methods/misc.rb'
require 'environment_information/toplevel_methods/replay_from_the_stored_file.rb'
# =========================================================================== #
# Add colours-support next:
# =========================================================================== #
require 'environment_information/colours/colours.rb'
require 'environment_information/toplevel_methods/e.rb'
require 'environment_information/toplevel_methods/is_on_roebe.rb'
require 'environment_information/toplevel_methods/register_this_component_is_missing.rb'
require 'environment_information/toplevel_methods/return_remote_gtk2_version.rb'

module EnvironmentInformation

  begin
    require 'rbt/toplevel_methods/swift_return_version_of_this_program.rb'
  rescue LoadError; end

  # ========================================================================= #
  # === EnvironmentInformation.show_all_available_components
  #
  # This method will quickly and easily show all available (registered)
  # components. If you need more fine-tuning when you wish to display
  # the main dataset at hand, then you should use other methods that
  # are also a bit more complex. This method here is really just a
  # fast version-display overview.
  #
  # The second argument to this method contains the programs that we
  # wish to display through this method.
  # ========================================================================= #
  def self.show_all_available_components(
      n_ljust               = 18,
      show_these_components = ARRAY_DEFAULT_PROGRAMS_ON_LINUX+
                              ARRAY_TRACKED_PROGRAMS,
      use_colours           = use_colours?
    )
    clear_main_hash
    if show_these_components.is_a? Array
      show_these_components.flatten!
    end
    # ======================================================================= #
    # The next variable determines whether we will also compare the
    # program versions.
    # ======================================================================= #
    compare_program_versions = false
    default_nljust_value = 30 # ← Detemine default padding.
    if n_ljust.is_a?(Array) and n_ljust.empty?
      n_ljust = default_nljust_value # Use the default value in this case.
    end
    # ======================================================================= #
    # First we will treat n_ljust as a commandline-flag if it is a String
    # and additionally begins with "--".
    # ======================================================================= #
    if n_ljust.is_a?(Array) and n_ljust.first.start_with?('--')
      _ = n_ljust.first
      case _
      # ===================================================================== #
      # === --compare-to-program-versions
      #
      # To invoke this, try:
      #
      #   show_components --compare
      #   envi --compare-version
      #
      # ===================================================================== #
      when *ARRAY_COMPARE_PROGRAM_VERSIONS
        compare_program_versions = true
        n_ljust = default_nljust_value
      # ===================================================================== #
      # === --xorg
      # ===================================================================== #
      when /^-?-?xorg$/i
        show_these_components = EnvironmentInformation.xorg_components?
        n_ljust = default_nljust_value
      # ===================================================================== #
      # === --help
      # ===================================================================== #
      when /^-?-?help$/i
        e 'The following options are available for bin/fast_envi (fenvi):'
        e
        e '   --xorg                        # show only the xorg-components'
        e '   --compare-to-program-versions # also compare the program versions'
        e
        exit
      end
    end
    e
    uniq = show_these_components.uniq
    uniq.sort.each {|this_program|
      use_this_name_for_send = "return_version_of_#{this_program}"
      # ===================================================================== #
      # Next, add the name of the program at hand, onto the left hand side:
      # ===================================================================== #
      result = this_program.to_s.ljust(n_ljust)
      if use_colours
        result = ::EnvironmentInformation.colourize_and_pad_the_left_side(result)
      end
      # ===================================================================== #
      # Some components may not be installed on the user's computer system,
      # which we have to keep in mind in the following code. This is
      # why we will first apply the .send(), before checking whether
      # the program at hand is actually missing or not.
      # ===================================================================== #
      result_of_send = send(use_this_name_for_send)
      if result_of_send
        result_of_send = result_of_send.dup if result_of_send.frozen?
        result_of_send.strip!
      end
      @hash_available_programs[this_program.to_sym] = result_of_send.to_s
      right_component = ''.dup
      if @array_this_component_is_missing.include?(this_program.to_sym)
        right_component << "not installed (or otherwise not found)".ljust(12)
      else
        right_component << result_of_send.ljust(12)
      end
      if use_colours
        # =================================================================== #
        # We will distinguish between components that have been found and
        # components that have not been found.
        # =================================================================== #
        if right_component.include?('not installed')
          right_component = ::Colours.send(:mediumslateblue, right_component)
        else
          right_component = ::Colours.send(colour_for_the_right_side, right_component)
        end
      end
      result = result.dup if result.frozen?
      result << right_component
      if compare_program_versions
        if is_this_program_included?(this_program)
          registered_local_version = RBT.swift_return_version_of_this_program(this_program.to_sym).to_s.dup
          if registered_local_version.include? 'v'
            # =============================================================== #
            # Modify entries such as 'v12.15.0' into '12.15.0'.
            # =============================================================== #
            registered_local_version.delete!('v')
          end
          case this_program.to_s
          when 'gtk2'
            registered_local_version = return_remote_gtk2_version
          else
            if result_of_send.nil?
              # ^^^ This is missing, then, so it will be ignored.
            elsif registered_local_version <= result_of_send 
              # This is ok.
            else
              result << 
                royalblue(
                  "\n    "\
                  "^^^ This could be updated; the version in "\
                  "RBT is: #{mediumpurple(registered_local_version)}"
                )
            end
          end
        end
      end
      # ===================================================================== #
      # Finally display our findings to the end user.
      # ===================================================================== #
      e result
    }
    e
    if is_on_roebe? # And store it on my home system too.
      ::EnvironmentInformation.store_relevant_files
    end
  end

end

if __FILE__ == $PROGRAM_NAME
  EnvironmentInformation.show_all_available_components(ARGV)
end # show_components --compare