#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === EnvironmentInformation::Sinatra
# =========================================================================== #
# require 'environment_information/www/sinatra_interface.rb'
# =========================================================================== #
require 'sinatra/base.rb'
require 'beautiful_url'
require 'environment_information/requires/require_the_environment_information_project.rb'

module EnvironmentInformation

class Sinatra < Sinatra::Base

  include Colours
  include Colours::E
  begin
    require 'html_tags'
    include HtmlTags::BaseModule
  rescue LoadError; end

  # ========================================================================= #
  # === USE_THIS_PORT
  # ========================================================================= #
  USE_THIS_PORT = '4567'

  set :port, USE_THIS_PORT

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize
    super()
    reset
    if ::EnvironmentInformation.is_on_roebe?
      require 'open_in_browser'
      # ===================================================================== #
      # Tell us which port will be used:
      # ===================================================================== #
      target = "http://localhost:#{USE_THIS_PORT}/"
      e sfancy(target)
      Thread.new {
        sleep 1.0
        OpenInBrowser[target]
      }
    end
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
  end

  # ========================================================================= #
  # === html_end
  # ========================================================================= #
  def html_end
    '<idv></body></html>'
  end

  # ========================================================================= #
  # === return_html_start
  #
  # This method will include all parts up to the first main <div>
  # element of the webpage.
  # ========================================================================= #
  def return_html_start
    "<html><head><title>\n"\
    "Sinatra-Interface of the EnvironmentInformation project\n"\
    "</title></head><body>"\
    "<div style=\"padding:2px; margin-left:0.5em\">"
  end

  # ========================================================================= #
  # === / Entry point
  #
  # This is the root of our web-application.
  # ========================================================================= #
  get('/') {
    # ======================================================================= #
    # We will initialize the main Hash next, that is kept in
    # EnvironmentInformation.
    # ======================================================================= #
    ::EnvironmentInformation.build_up_and_return_the_main_hash
    inner_content = 
      h2('Displaying information about the local computersystem')+
      uniform_p_tag('Operating system in use:',
        ::EnvironmentInformation.operating_system?)+
      uniform_p_tag('OS Bit type (architecture):', 
        ::EnvironmentInformation.bit_type?)+
      uniform_p_tag('CPU Model:', 
        ::EnvironmentInformation.cpu_model?)+
      uniform_p_tag('CFLAGS in use:', 
        ::EnvironmentInformation.cflags_in_use?)+
      uniform_p_tag('RAM (in MB):', 
        ::EnvironmentInformation.ram?)+
      uniform_p_tag('Screen resolution:',
        ::EnvironmentInformation.screen_resolution?.to_s)
    inner_content = inner_content.dup
    # ======================================================================= #
    # Next, add the individual components:
    # ======================================================================= #
    %i(
      gcc
      glibc
      linux_kernel
      binutils
      awk
      ruby
      rubygems
      coreutils
      diffutils
      bison
      boost
      cmake
      file
      flex
      intltool
      gettext
      grep
      gnupg
    ).each {|this_component|
      inner_content << sinatra_paragraph_showing_this_component(this_component).to_s
    }
    return_html_start+
    div(css_style: 'font-size: normal; margin-top:1px; margin-bottom: 1px;') {
      inner_content
    }+
    html_end
  }

  # ========================================================================= #
  # === uniform_p_tag
  # ========================================================================= #
  def uniform_p_tag(i, content)
    i = i.to_s
    unless i.end_with? ':'
      i = i.dup if i.frozen?
      i << ':'
    end
    p(
      "#{i}<br><b style=\"color: darkblue; margin-left:1.5em\">#{content}"\
      "</b>",
      css_style: 'margin-left: 2.5em;
                  margin: 4px;
                  padding: 5px;
                  padding-top: 0.2em'
    )
  end

  # ========================================================================= #
  # === sinatra_paragraph_showing_this_component
  #
  # This method was added primarily so that we can use a unified (and
  # consistent) style for display.
  # ========================================================================= #
  def sinatra_paragraph_showing_this_component(
      i = :gcc
    )
    content = "return_version_of_#{i}".to_s
    content = ::EnvironmentInformation.send(content)
    # ======================================================================= #
    # Next, we grab it from the main Hash.
    # ======================================================================= #
    case i
    when :awk,
         :bison,
         :boost,
         :coreutils,
         :binutils,
         :cmake,
         :diffutils,
         :findutils,
         :file,
         :flex,
         :gcc,
         :gettext,
         :glibc,
         :grep,
         :gnupg,
         :intltool,
         :ruby,
         :rubygems
      i = i.to_s.capitalize+' Version'
    when :linux_kernel
      i = 'Linux Kernel'
    end
    # ======================================================================= #
    # Finally return the proper <p> tag:
    # ======================================================================= #
    uniform_p_tag(i, content)
  end

end

# =========================================================================== #
# === EnvironmentInformation.start_sinatra_interface
#
# This method can be used to start the sinatra interface.
# =========================================================================== #
def self.start_sinatra_interface
  puts 'Trying to start the sinatra-interface of EnvironmentInformation next.'
  ::EnvironmentInformation::Sinatra.run!
end

end

if __FILE__ == $PROGRAM_NAME
  EnvironmentInformation.start_sinatra_interface
end # envi --sinatra